

//*****************************************************************************
//
//  Copyright(c) 2016 Synaptic Laboratories Limited (Synaptic Labs)
//
//  All rights reserved.
//
//  This document contains confidential and proprietary product to Synaptic Labs.
//
//  No part of this file may be copied, reproduced, disclosed or provided to
//  third parties without prior written consent of Synaptic Labs.
//
//  Synaptic Labs reserves the right to make changes in the content of this
//  file without prior notice.
//
//*****************************************************************************
//
//*****************************************************************************
//
//  THIS Verilog MODULE IS INTENDED FOR USE IN COMMERCIAL PRODUCTS.
//
//*****************************************************************************
//
//*****************************************************************************
//
//  Title          :  SLL Hyperbus Controller Input/Ouput Pads
//  Project        :  N/A
//
//  File           :  sll_ca_hbc_t001_io.v
//  Author         :  Mark Bonnici
//
//  Email          :  mark.bonnici@synaptic-labs.com
//  Revision date  :  Feb/15/2017
//
//  Description    : SLL Hyperbus Controller Input/Ouput Pads, PLL, clock generation
//
//
//  Known issues   :  None
//
//  References     :
//
//*********************************************************************************************************************************
//  Revision     Date         Author         Comments
//
//  Version 1.0  05/Jan/2017   Mark     Initial Release
//  Version 1.1  15/Feb/2017   Mark     Re-arrranged interface, added comments
//  Version 2.10 22/Mar/2017   Mark     Addded g_device_family parameter
//  Version 2.11 28/Apr/2017   Mark     changed parameters for synpll module
//*********************************************************************************************************************************
`timescale 1ps/1ps


module sll_ca_hbc_t001_io (
	 in_clk
	,ext_nrst
	
	,clk_0
	,reset_n
	
	,clk_core

//Hyper Bus Cotroller Signals
  ,ck_en
  ,csn_en
  ,csn_d

  ,rds_clk
  ,rwds_in
  ,rwds_io_tri
  ,wds0
  ,wds1
  ,wds_en
  
  ,dq_in_en
  ,dq_io_tri
  ,dq_out_en
  ,dq_out0
  ,dq_out1
  ,dq_in
   
//Hyper Bus Device Siganls
	,HB_CLK0n
	,HB_CLK1
	,HB_CLK1n
	,HB_CLK0
	,HB_CSn
	,HB_RWDS
	,HB_dq
);
parameter g_input_clk_freq_in_mhz = 50;          //Input Pll Clock Frequency (in Megahertz)
parameter g_hyperbus_freq_in_mhz  = 100;         //Requested Hyperbus Clock operational Frequency (in Megahertz)
parameter g_core_freq_in_mhz      = 100;         //Requested Avalon-MM Clock operational Frequency (in Megahertz)
parameter g_dqin_rdata_width      = 8;           //Read data width
parameter g_num_chipselect        = 2;           //Number of chips selects (only 1 or 2 are supported)
parameter g_device_family         = "MAX 10";   

//Pll Signals  
input  wire	                       in_clk;       //input clock top PLL
input  wire	                       ext_nrst;     //Input external reset (active low)
output wire	                       reset_n;      //Output reset to other logic (active low)
                                                 
output                             clk_0;        //output hyperbus clock  to hyperbus core controller
output                             clk_core;     //output avalon-mm clock to avalon--MM controller
                                                 
//Hyper Bus Device Siganls                       
output wire	                       HB_CLK0;      //Output Clock 0
output wire	                       HB_CLK0n;     //Output Clock 0 (inverted)
output wire	                       HB_CLK1;      //Output Clock 1
output wire	                       HB_CLK1n;     //Output Clock 0 (inverted)
output wire	[g_num_chipselect-1:0] HB_CSn;       //Output Chips Select (acive low)
inout wire	                       HB_RWDS;      //Input/output read strobe/Write Mask
inout wire	 [7:0]                 HB_dq;        //Input/Output data
                                                 
//Signals to the Core                            
output                             rds_clk;      //read strobe clock
output[g_dqin_rdata_width-1:0]     dq_in  ;      //Data in 
output                             rwds_in;      //refresh indicator

//Signals from the Core (clocked by clk_0)
input [g_num_chipselect-1:0]        csn_en;      //Enable Chip select
input                               csn_d;       //Set/clear Chip select
input                               ck_en;       //Toggle Clock State
input                               dq_in_en;    //data in enable input (currently not used)
input                               dq_io_tri;   //data out tristate enable 
input                               dq_out_en;   //data out register enable
input [7:0]                         dq_out0;     //data out lower byte
input [7:0]                         dq_out1;     //data out upper byte
input                               wds_en;      //write mask enable
input                               wds0;        //write mask lower byte
input                               wds1;        //write mask upper byte
input                               rwds_io_tri; //Write Mask data tri-state enable




wire	GND;
wire	VCC;

assign	GND = 1'b0;
assign	VCC = 1'b1;

//------------------------------------
//pll signals and control
//------------------------------------
wire	clk_0    /* synthesis keep */;
wire	clk_180  /* synthesis keep */;
wire	clk_270  /* synthesis keep */;
wire	clk_90   /* synthesis keep */;
wire	clk_core;
wire pll_locked, reset;

  sll_ca_hbc_t001_syncpll #(
     .g_input_clk_freq_in_mhz(g_input_clk_freq_in_mhz)
    ,.g_hyperbus_freq_in_mhz (g_hyperbus_freq_in_mhz)
    ,.g_core_freq_in_mhz     (g_core_freq_in_mhz)
    ,.g_device_family        (g_device_family )  
  ) U_SLL_HBC_T001_PLL (
	   .inclk0(in_clk   )
	  ,.areset(~ext_nrst)
	  ,.c0    (clk_0    )
	  ,.c1    (clk_90   )
	  ,.c2    (clk_180  )
	  ,.c3    (clk_270  )
	  ,.c4    (clk_core )
	  ,.locked(pll_locked)
	);

  assign reset_n =  ext_nrst &  pll_locked;	
  assign reset   = ~reset_n; 	  
	
//------------------------------------
//dq out
//
//register with clk_270
//------------------------------------   
  wire[7:0]  dq_in;
  reg	       dq_out_en_270_r;
  reg	[15:0] dq_out_270_r;

  always@(posedge clk_270)
  begin
  	dq_out_en_270_r    <=  dq_out_en;
  	dq_out_270_r[15:0] <= {dq_out1, dq_out0};
  end

   
  //ddio output Pad in DDR mode 
  sll_ca_hbc_t001_ddio_out #(
    .width(8)
  )	U_DQ (
  	 .outclock  (clk_90         )
  	,.outclocken(dq_out_en_270_r)
   	,.aset      (1'b0           )
  	,.aclr      (reset          )
  	,.din       (dq_out_270_r   )
  	,.oe        ({8{~dq_io_tri}})
  	,.pad_out   (HB_dq          )
  );
  
  assign	dq_in = HB_dq;

//------------------------------------
//wds - write mask strobe
//
//register with clk_270
//------------------------------------   
   reg	      wds_en_270_r;
   reg	[1:0] wds_270_r;

   always@(posedge clk_270)
   begin
    	wds_en_270_r   <=  wds_en;
   	  wds_270_r[1:0] <= {wds1, wds0};
   end

  //ddio output Pad in DDR mode 
   sll_ca_hbc_t001_ddio_out #(
    .width(1)
  	) U_RWDS (
   	 .outclock  (clk_90      )
   	,.outclocken(wds_en_270_r)
   	,.aclr      (reset       )
   	,.aset      (1'b0        )
   	,.oe        (~rwds_io_tri)
   	,.din       (wds_270_r   )
   	,.pad_out   (HB_RWDS     )
   );

//------------------------------------
//RDW signal
//------------------------------------
  wire  rwds_in_delayed;
  wire	rwds_in;
  wire  rds_clk /* synthesis keep */;

  //actual signal is delayed using the PAD_TO_CORE_DELAY constraint
  assign	#(700) rwds_in_delayed = HB_RWDS;//delay of 700 ps

`ifdef INCLUDE_VAR_LATENCY
  //refresh indicator to core
  reg rwds_in_0;
   always @(posedge clk_0) 
   begin
   	  rwds_in_0 <= rwds_in_delayed;
   end
   
  assign rwds_in = rwds_in_0 ;
`else
  assign rwds_in = 1'b1 ;
`endif
  
  
  //read strobe to Core
  assign rds_clk = rwds_in_delayed; 

//------------------------------------
//CLK outputs
//
//For CLK0, CLK1
//   Keep clk low when ck_en is NOT set
//   Toggle clock when ck_en is set
//
//For CLK0n, CLK1n
//   Keep clk High when ck_en is NOT set
//   Toggle clock when ck_en is set
//
//------------------------------------
   wire [1:0] clk_mux;
   wire [1:0] clk_n_mux;
   
   assign clk_mux   =  (ck_en) ? {GND,VCC} : {GND,GND};
   assign clk_n_mux = ~clk_mux;
   
  //ddio output Pad in DDR mode 
   sll_ca_hbc_t001_ddio_out #(
    .width     (1)
   ,.rst_high  (0)
   ,.en_oe_port(0)
  	) U_CLK0 (
   	 .outclock  (clk_180  )
   	,.outclocken(1'b1     )
   	,.aclr      (reset    )
   	,.aset      (1'b0     )
   	,.oe        (1'b0     )
   	,.din       (clk_mux  )
   	,.pad_out   (HB_CLK0  )
   );


   sll_ca_hbc_t001_ddio_out #(
    .width     (1)
   ,.rst_high  (1)
   ,.en_oe_port(0)
  	) U_CLK0n (
   	 .outclock  (clk_180  )
   	,.outclocken(1'b1     )
   	,.aclr      (1'b0     )
   	,.aset      (reset    )
   	,.oe        (1'b0     )
   	,.din       (clk_n_mux)
   	,.pad_out   (HB_CLK0n )
   );

  sll_ca_hbc_t001_ddio_out #(
    .width     (1)
   ,.rst_high  (0)
   ,.en_oe_port(0)
  	) U_CLK1 (
   	 .outclock  (clk_180  )
   	,.outclocken(1'b1     )
   	,.aclr      (reset    )
   	,.aset      (1'b0     )
   	,.oe        (1'b0     )
   	,.din       (clk_mux  )
   	,.pad_out   (HB_CLK1  )
   );


   sll_ca_hbc_t001_ddio_out #(
    .width     (1)
   ,.rst_high  (1)
   ,.en_oe_port(0)
  	) U_CLK1n (
   	 .outclock  (clk_180  )
   	,.outclocken(1'b1     )
   	,.aclr      (1'b0     )
   	,.aset      (reset    )
   	,.oe        (1'b0     )
   	,.din       (clk_n_mux)
   	,.pad_out   (HB_CLK1n )
   );
   
//------------------------------------
//csn outputs
//
//At reset CSNx is set high
//Otherwise CSNx is set and cleared 
//according to the value of csn_en and csn_d
//     _______                 _______________
// CSn        |_______________|
//            _              _ 
// csn_en____| |___________ | |__
//            _
// csd   ____| |__________________
//
//------------------------------------
   genvar csn_loop;
   generate  
   for (csn_loop=0; csn_loop < g_num_chipselect; csn_loop=csn_loop+1) begin : csn_
   	  reg       csn_mux_h_r;
      wire[1:0] csn_mux;

      always @(posedge clk_0 or negedge reset_n)
      begin
      	  if (reset_n == 1'b0) begin
      	    csn_mux_h_r <= 1'b1;	
        	end else begin
      	    csn_mux_h_r <= csn_mux[1];	
        	end
      end
      assign csn_mux   = (csn_en[csn_loop]) ? {csn_d,GND} : {csn_mux_h_r, csn_mux_h_r};

     //ddio output Pad in DDR mode 
      sll_ca_hbc_t001_ddio_out #(
       .width     (1)
      ,.rst_high  (1)
      ,.en_oe_port(0)
  	   ) U_CS1n (
      	 .outclock  (clk_0  )
      	,.outclocken(1'b1     )
      	,.aset      (reset    )
      	,.aclr      (1'b0     )
      	,.oe        (1'b0     )
      	,.din       (csn_mux)
      	,.pad_out   (HB_CSn[csn_loop] )
      );  
   end   
  endgenerate

endmodule
